/*
								+---------------------------------+
								|                                 |
								|    ***   Shader system   ***    |
								|                                 |
								|  Copyright   -tHE SWINe- 2006  |
								|                                 |
								|           ShaderSys.h           |
								|                                 |
								+---------------------------------+
*/

/*
 *	passed code revision at 02-05-2007
 *
 *	fixed some signed / unsigned glitches
 *	removed const_iterator to iterator type cast hack in CGLShaderObject::n_TextureType()
 *	and in CGLShaderObject::n_TextureUnit()
 *
 */

#ifndef __SHADER_SYSTEM_INCLUDED
#define __SHADER_SYSTEM_INCLUDED

/*
<package name="package_name">
	<shader name="name" type="[vertex|fragment]">
		<high>
			<sampler tex_unit="[0-7]">sampler_name</sampler>
			<sampler tex_unit="[0-7]">sampler_name</sampler>
			<sampler tex_unit="[0-7]">sampler_name</sampler>
			<!-- in case tex_unit is not specified for certain sampler, it's assigned
			next texture unit in line -->

			[<src>void main() { ftransform(); }</src>|<src file="filename.glsl"/>]
		</high>
		<low>
			<env_param register="[0-31]">param_name</env_param>
			<local_param register="[0-31]">param_name</env_param>
			<local_param register="[0-31]">param_name</env_param>

			[<src>!!ARBvp1.0 OPTION ARB_position_invariant; END</src>|<src file="filename.arbsl"/>]
		</low>
	</shader>
</package>

<material name="name" fill_expensive="[true|false]" transform_expensive="[true|false]"
   transparent="[true|false]" pack_coplanar="[true|false]" fill_mirror="[true|false]">
	<path names="name0 name1 'name 2'" vendors="ati nv *"
	   models="[+|-]card_name0 '[+|-]card name 1'" extensions="ext0 ext1">
		<pass purpose="[ambient|diffuse_specular|depth]" [no_draw="true"]> <!-- no_draw se pouzije pokud se pouziji combinery, atd - potom je to jen vypne ale neposila zadnou geometrii -->
			<texture src="">
				<!-- parameters, coord generation, enviroment -->
			</texture>
			<!-- rasterizer state, alpha-test, depth-test, write mask, blending, polygon offset -->
			<shader vertex=[""|"package_name/name"] fragment=[""|"package_name/name"]>
				<param name="param_name[.swizzle]"><!-- param value (constant / system variable name) --></param>
				<param name="param_name[.swizzle]"><!-- param value (constant / system variable name) --></param>
				<param name="param_name[.swizzle]"><!-- param value (constant / system variable name) --></param>
			</shader>
		</pass>
	</path>
</material>

  (xml notation)

namespace PackageName; // if not present, shaders are added into glogal namespace

[vertex|fragment] shader [V|F]SName {
high:
	sampler {
		sampler_name = [0-7],
		sampler_name = [0-7],
		sampler_name = [0-7]
	};
	// in case texture unit number is not specified for certain sampler, it's assigned
	// next texture unit in line

	[source = "void main() { ftransform(); }";|source_file = "filename.glsl";]

low:
	env_param {
		param_name [= [0-31]],
		param_name [= [0-31]],
		param_name [= [0-31]]
	};

	local_param {
		param_name [= [0-31]],
		param_name [= [0-31]],
		param_name [= [0-31]]
	};
	// in case register number is not specified, it's assigned
	// next register in line

	[source = "!!ARBvp1.0 OPTION ARB_position_invariant; END";|source_file = "filename.arbsl";]
};

[transform_expensive] [fill_expensive] [transparent] [pack_coplanar] [fill_mirror] material MName {
	path PName : alias PName2, PName3 {
		vendor {
			"vendor0",
			"vendor1",
			"*"
		};

		model {
			"[+|-]model name0",
			"[+|-]model name1",
			"[+|-]model name2"
		};

		extension {
			"GL_ARB_ext_name0",
			"GL_ARB_ext_name1",
			"GL_ARB_ext_name2"
		};

		[ambient|diffuse_specular|depth] [no_draw] pass {
			// ...
		};
	};
};

  (c++ like notation)
*/

/*
 *	class CStringCompare
 *		- util class for string searches using std::find_if
 */
class CStringCompare {
protected:
	const char *m_p_s_str;
public:
	__inline CStringCompare(const char *p_s_str)
		:m_p_s_str(p_s_str)
	{ }

	__inline bool operator ()(const char *p_s_str) const
	{
		return !strcmp(p_s_str, m_p_s_str);
	}
};

/*enum {
	shader_param_Uniform, // ARB_*_shader unifrom
	shader_param_Parameter_Env, // ARB_*_program env param
	shader_param_Parameter_Local_VP, // ARB_vertex_program local param
	shader_param_Parameter_Local_FP // ARB_fragment_program local param
};*/ // Shader2.h

/*
 *	struct TShaderInfo
 *		- contains informations on shader, loaded from .xml or .s++ file
 */
struct TShaderInfo {
	bool b_fragment; // false = vertex processor, true = fragment processor
	bool b_shader_target; // false = ARB_*_program, true = ARB_*_shader
	char *p_s_name;

	struct TNamedResource {
		char *p_s_name;
		int n_resource_index;

		__inline TNamedResource()
			:p_s_name(0)
		{}

		__inline ~TNamedResource()
		{
			if(p_s_name)
				delete[] p_s_name;
		}
	};

	struct TShader {
		typedef TNamedResource TSamplerInfo;
		TSamplerInfo *p_sampler; // used for explicit texture unit assignment only; texture samplers are located automatically by OpenGL
		int n_sampler_num;

		char *p_s_source_code;

		__inline TShader()
			:p_s_source_code(0), p_sampler(0)
		{}

		__inline ~TShader()
		{
			if(p_s_source_code)
				delete[] p_s_source_code;
			if(p_sampler)
				delete[] p_sampler;
		}
	} *p_high;

	struct TProgram {
		struct TParamInfo : public TNamedResource {
			bool b_enviroment;
		} *p_parameter;
		int n_parameter_num;

		typedef TNamedResource TTexUnitInfo;
		TTexUnitInfo *p_texture_unit; // used for texture unit naming
		int n_texture_unit_num;

		char *p_s_source_code;

		__inline TProgram()
			:p_s_source_code(0), p_parameter(0)
		{}

		__inline ~TProgram()
		{
			if(p_s_source_code)
				delete[] p_s_source_code;
			if(p_parameter)
				delete[] p_parameter;
		}
	} *p_low;

	TShaderInfo()
		:p_s_name(0), p_high(0), p_low(0)
	{}

	~TShaderInfo()
	{
		if(p_s_name)
			delete[] p_s_name;
		if(p_high)
			delete[] p_high;
		if(p_low)
			delete[] p_low;
	}

	/*
	 *	bool Dynamize()
	 *		- presumably, the structure was filled by copying from another one or by const data
	 *		- this function allocates new buffers for shader info and, copies current values
	 *		  inside and then assigns those new buffers to be used by this shader info
	 *		- returns true on success, false on failure (presumably not enough memory)
	 */
	bool Dynamize();

	static char *p_s_Alloc(const char *p_s_str);

	static TShaderInfo *p_CreateShaderInfo(const char *p_s_source); // todo - create a new shader info from text source (source is in .h file as constant)
};

class CGLShaderObject;

template <class TScalarType>
class CShaderParameterRef { friend CGLShaderObject;
protected:
	TScalarType *m_p_value; // target value
	int m_n_size;
	CGLShaderObject *m_p_parent;

public:
	/*
	 *	CShaderParameterRef::CShaderParameterRef(TScalarType *p_value,
	 *		int n_size, CGLShaderObject *p_parent)
	 *		- default constructor
	 *		- p_value is destination value
	 *		- n_size is array size (must be 1 to 4)
	 */
	CShaderParameterRef(TScalarType *p_value, int n_size, CGLShaderObject *p_parent)
		:m_p_value(p_value), m_n_size(n_size), m_p_parent(p_parent)
	{
		_ASSERTE(p_value && n_size >= 1 && n_size <= 4 && p_parent);
	}

	/*
	 *	CShaderParameterRef::~CShaderParameterRef()
	 *		- default destructor
	 *		- invalidates itself automatically
	 */
	~CShaderParameterRef()
	{
		m_p_parent->Remove_Parameter(this, false);
	}

	/*
	 *	bool CShaderParameterRef::b_Valid() const
	 *		- returns true in case parameter is valid (shader it points to is valid)
	 */
	bool b_Valid() const
	{
		return m_p_value != 0;
	}

	/*
	 *	int n_Size() const
	 *		- returns size of parameter (1 = scalar, 2 to 4 = vector or square matrix)
	 */
	int n_Size() const
	{
		return m_n_size;
	}

private:
	/*
	 *	bool CShaderParameterRef::Invalidate(bool b_remove = true)
	 *		- invalidates this parameter (used by CGLShaderObject; it keeps track of
	 *		  all it's parameters and in case it's destroyed / recompiled, it invalidates
	 *		  all parameters)
	 *		- always returns true (needed for std::mem_fun)
	 */
	bool Invalidate()
	{
		if(m_p_value) {
			//m_p_parent->Remove_Parameter(this, false);
			m_p_value = 0;
			m_p_parent = 0;
		}
		return true;
	}
};

class CSwizzleGenerator {
public:
	enum {
		swz_X = 1,
		swz_XY = (2 << 3) + 1,
		swz_XYZ = (3 << 6) | (2 << 3) | 1,
		swz_XYZW = (4 << 9) | (3 << 6) | (2 << 3) | 1,
	};
	// some common swizzles

	/*
	 *	static int CSwizzleGenerator::n_Swizzle(const char *p_s_swizzle)
	 *		- text to integer translation for n_swizzle operand in SetValue()
	 *		- p_s_swizzle is text swizzle operand in format [xyzw][[xyzw]][[xyzw]][[xyzw]]
	 *		  and enables input value reordering to destination vector
	 *		  (for example for two input values 1 and 2 and swizzle xyxx output is set to 1211)
	 *		- returns -1 on failure (operands with more than 4 characters or operands with
	 *		  invalid characters)
	 */
	static int n_Swizzle(const char *p_s_swizzle)
	{
		if(strlen(p_s_swizzle) > 4 || !strlen(p_s_swizzle))
			return -1;

		int n_result = 0;
		for(int n_shift = 0; *p_s_swizzle; p_s_swizzle ++, n_shift += 3) {
			if(*p_s_swizzle == '-')
				n_result += 4 << n_shift;
			else {
				char n_component = *p_s_swizzle | 32;
				if(n_component < 'w' || n_component > 'z')
					return -1;
				n_result += (((n_component - 'x') & 3) + 1) << n_shift;
			}
		}
		// 12 lowest bits used for components, every 3 bits contain component index (1 - 4) or
		// 0 = leave current value

		return n_result;
	}
};

template <class TScalarType>
class CShaderVectorParameterRef : public CShaderParameterRef<TScalarType>, CSwizzleGenerator {
friend CGLShaderObject;
public:
	/*
	 *	__inline CShaderVectorParameterRef::CShaderVectorParameterRef(TScalarType *p_value,
	 *		int n_size, CGLShaderObject *p_parent)
	 *		- default constructor
	 *		- p_value is destination value
	 *		- n_size is array size (must be 1 to 4)
	 */
	__inline CShaderVectorParameterRef(TScalarType *p_value, int n_size, CGLShaderObject *p_parent)
		:CShaderParameterRef<TScalarType>(p_value, n_size, p_parent)
	{ }

	/*
	 *	__fastcall bool CShaderVectorParameterRef::SetValue(const TScalarType *p_value,
	 *		int n_swizzle)
	 *		- set target parameter value
	 *		- p_value is source value
	 *		- n_swizzle is swizzle operand used to reorder / mask input values; p_value
	 *		  must contain as much (1 to 4) scalars as needed by swizzle op
	 *		- returns true on success, false in case n_swizzle is invalid swizzle operand
	 *		  or shader parameter is no longer valid
	 */
	bool __fastcall SetValue(const TScalarType *p_value, int n_swizzle)
	{
		if(!m_p_value)
			return false;
		
		for(int i = 0; i < m_n_size; i ++, n_swizzle >>= 3) {
			if(n_swizzle & 7)
				m_p_value[i] = p_value[(n_swizzle & 7) - 1];
		}
		// swizzle / copy parameters

		return !n_swizzle;
		// in case n_swizzle is nonzero, intended swizzle output is longer than parameter size
	}
};

class CShaderMatrixParameterRef : public CShaderParameterRef<float> {
public:
	/*
	 *	__inline CShaderMatrixParameterRef::CShaderMatrixParameterRef(float *p_value,
	 *		int n_size, CGLShaderObject *p_parent)
	 *		- default constructor
	 *		- p_value is destination value
	 *		- n_size is array size (must be 2 to 4)
	 */
	__inline CShaderMatrixParameterRef(float *p_value, int n_size, CGLShaderObject *p_parent)
		:CShaderParameterRef<float>(p_value, n_size, p_parent)
	{ }

	/*
	 *	__fastcall bool CShaderMatrixParameterRef::SetValue(const float *p_value,
	 *		bool b_transpose = false)
	 *		- set target parameter value
	 *		- p_value is source value
	 *		- returns true on success, false in case shader parameter is no longer valid
	 */
	bool __fastcall SetValue(const float *p_value, bool b_transpose = false)
	{
		if(!m_p_value)
			return false;

		memcpy(m_p_value, p_value, m_n_size * m_n_size * sizeof(float));

		return true;
	}
};

/*enum {
	shader_param_Float = GL_FLOAT,
	shader_param_Float2 = GL_FLOAT_VEC2_ARB,
	shader_param_Float3 = GL_FLOAT_VEC3_ARB,
	shader_param_Float4 = GL_FLOAT_VEC4_ARB,
	shader_param_Int = GL_INT,
	shader_param_Int2 = GL_INT_VEC2_ARB,
	shader_param_Int3 = GL_INT_VEC3_ARB,
	shader_param_Int4 = GL_INT_VEC4_ARB,
	shader_param_Bool = GL_BOOL_ARB,
	shader_param_Bool2 = GL_BOOL_VEC2_ARB,
	shader_param_Bool3 = GL_BOOL_VEC3_ARB,
	shader_param_Bool4 = GL_BOOL_VEC4_ARB,
	shader_param_Matrix2 = GL_FLOAT_MAT2_ARB,
	shader_param_Matrix3 = GL_FLOAT_MAT3_ARB,
	shader_param_Matrix4 = GL_FLOAT_MAT4_ARB,
	shader_sampler_1D = GL_SAMPLER_1D_ARB,
	shader_sampler_2D = GL_SAMPLER_2D_ARB,
	shader_sampler_3D = GL_SAMPLER_3D_ARB,
	shader_sampler_Cube = GL_SAMPLER_CUBE_ARB,
	shader_sampler_1D_Shadow = GL_SAMPLER_1D_SHADOW_ARB,
	shader_sampler_2D_Shadow = GL_SAMPLER_2D_SHADOW_ARB,
	shader_sampler_2D_Rect = GL_SAMPLER_2D_RECT_ARB,
	shader_sampler_2D_Rect_Shadow = GL_SAMPLER_2D_RECT_SHADOW_ARB
};*/ // Shader2.h

struct TParHandle_Base {
	int n_target; // one of shader_param_Uniform, shader_param_Parameter_Env, shader_param_Parameter_Local_VP or shader_param_Parameter_Local_FP
	int n_register; // location in register bank (0-31)
	int n_type; // shader content type (one of shader_param_Float, shader_param_Float2, ...)
	char *p_s_name;

	__inline TParHandle_Base(int _n_target, int _n_register, int _n_type, char *_p_s_name)
		:n_target(_n_target), n_register(_n_register), n_type(_n_type), p_s_name(_p_s_name)
	{}

	__inline TParHandle_Base(const TParHandle_Base &r_param)
		:n_target(r_param.n_target), n_register(r_param.n_register), n_type(r_param.n_type)
	{
		if(p_s_name = new char[strlen(r_param.p_s_name) + 1])
			strcpy(p_s_name, r_param.p_s_name);
	}

	__inline TParHandle_Base operator =(const TParHandle_Base &r_param)
	{
		n_target = r_param.n_target;
		n_register = r_param.n_register;
		n_type = r_param.n_type;
		if(p_s_name)
			delete[] p_s_name;
		if(p_s_name = new char[strlen(r_param.p_s_name) + 1])
			strcpy(p_s_name, r_param.p_s_name);

		return *this;
	}

	__inline ~TParHandle_Base()
	{
		delete[] p_s_name; 
	}
};

class CParamHandleNameCompare {
protected:
	const char *m_p_s_name;
public:
	__inline CParamHandleNameCompare(const char *p_s_name)
		:m_p_s_name(p_s_name)
	{ }

	__inline bool operator ()(const TParHandle_Base &r_par_handle) const
	{
		return !strcmp(r_par_handle.p_s_name, m_p_s_name);
	}
};

struct TSamplerHandle : public TParHandle_Base {
	int n_texture_unit; // texture unit number

	__inline TSamplerHandle(int _n_target, int _n_register, int _n_type, char *_p_s_name)
		:n_texture_unit(-1), TParHandle_Base(_n_target, _n_register, _n_type, _p_s_name)
	{}

	__inline TSamplerHandle(const TSamplerHandle &r_param)
		:n_texture_unit(r_param.n_texture_unit), TParHandle_Base(r_param)
	{}

	__inline TSamplerHandle operator =(const TSamplerHandle &r_param)
	{
		n_texture_unit = r_param.n_texture_unit;
		TParHandle_Base::operator =(r_param);
		return *this;
	}

	__inline bool b_UnassignedTextureUnit() /*const*/ // can't be const becuse of use with std::mem_fun
	{
		return n_texture_unit == -1;
	}
};

struct TScalarParamHandle : public TParHandle_Base {
	union TValueUnion {
		int n_value;
		bool b_value;
		float f_value;
	} t_value;

	__inline TScalarParamHandle(int _n_target, int _n_register, int _n_type, char *_p_s_name)
		:TParHandle_Base(_n_target, _n_register, _n_type, _p_s_name)
	{
		memset(&t_value, 0, sizeof(TValueUnion));
	}

	__inline TScalarParamHandle(const TScalarParamHandle &r_param)
		:t_value(r_param.t_value), TParHandle_Base(r_param)
	{}

	__inline TScalarParamHandle operator =(const TScalarParamHandle &r_param)
	{
		t_value = r_param.t_value;
		TParHandle_Base::operator =(r_param);
		return *this;
	}
};

struct TVectorParamHandle : public TParHandle_Base {
	union TValueUnion {
		int p_int_value[4];
		float p_float_value[4];
	} t_value;

	__inline TVectorParamHandle(int _n_target, int _n_register, int _n_type, char *_p_s_name)
		:TParHandle_Base(_n_target, _n_register, _n_type, _p_s_name)
	{
		memset(&t_value, 0, sizeof(TValueUnion));
	}

	__inline TVectorParamHandle(const TVectorParamHandle &r_param)
		:t_value(r_param.t_value), TParHandle_Base(r_param)
	{}

	__inline TVectorParamHandle operator =(const TVectorParamHandle &r_param)
	{
		t_value = r_param.t_value;
		TParHandle_Base::operator =(r_param);
		return *this;
	}
};

struct TMatrixParamHandle : public TParHandle_Base {
	float p_value[16]; // up to 4x4 matrix

	__inline TMatrixParamHandle(int _n_target, int _n_register, int _n_type, char *_p_s_name)
		:TParHandle_Base(_n_target, _n_register, _n_type, _p_s_name)
	{
		memset(p_value, 0, 16 * sizeof(float));
	}

	__inline TMatrixParamHandle(const TMatrixParamHandle &r_param)
		:TParHandle_Base(r_param)
	{
		memcpy(p_value, r_param.p_value, 16 * sizeof(float));
	}

	__inline TMatrixParamHandle operator =(const TMatrixParamHandle &r_param)
	{
		memcpy(p_value, r_param.p_value, 16 * sizeof(float));
		TParHandle_Base::operator =(r_param);
		return *this;
	}
};

/*enum {
	shader_api_ARB_program,
	shader_api_ARB_shader,
	shader_api_Core_shader // OpenGL20
};*/ // Shader2.h

class CGLShaderObject { friend CShaderParameterRef<int>; friend CShaderParameterRef<float>;
protected:
	const TShaderInfo *m_p_vertex_shader;
	const TShaderInfo *m_p_fragment_shader;
	bool m_b_use_shader_target;

	CGLProgram *m_p_shader;

	std::vector<TVectorParamHandle> m_parameter_list; // float1 - float4 parameters only
	std::vector<TSamplerHandle> m_sampler_list; // sampler parameters
	std::vector<TVectorParamHandle> m_int_parameter_list; // int1 - int4 parameters only
	std::vector<TVectorParamHandle> m_bool_parameter_list; // bool1 - bool4 parameters only
	std::vector<TMatrixParamHandle> m_matrix_parameter_list; // matrix2 - matrix4 parameters only
	std::vector<CShaderParameterRef<float>*> m_parameter_ref_list;
	std::vector<CShaderParameterRef<int>*> m_int_bool_parameter_ref_list;

public:
	/*
	 *	CGLShaderObject::CGLShaderObject(const TShaderInfo *m_p_vertex_shader,
	 *		const TShaderInfo *m_p_fragment_shader)
	 *		- default constructor
	 *		- m_p_vertex_shader and m_p_fragment_shader holds shader source code,
	 *		  one of them can be 0
	 *		- pointers are going to be referenced only, objects they point must remain
	 *		  allocated till destruction of this object
	 */
	CGLShaderObject(const TShaderInfo *m_p_vertex_shader, const TShaderInfo *m_p_fragment_shader);

	/*
	 *	CGLShaderObject::~CGLShaderObject()
	 *		- default destructor
	 */
	~CGLShaderObject();

	/*
	 *	const char *CGLShaderObject::p_s_VertexShader_Name()
	 *		- returns name of vertex shader if present or 0 if not present
	 */
	const char *p_s_VertexShader_Name();

	/*
	 *	const char *CGLShaderObject::p_s_FragmentShader_Name()
	 *		- returns name of fragment shader if present or 0 if not present
	 */
	const char *p_s_FragmentShader_Name();

	/*
	 *	bool CGLShaderObject::b_ShaderSupport(bool b_shader_target) const
	 *		- returns true in case extensions necessary to compile supplied
	 *		  shaders is available (in case vertex shader only was supplied,
	 *		  vertex shader support is sufficient, the same applies for fragment shader)
	 *		- b_shader_target specifies shader language (low-level program;
	 *		  b_shader_target = false or high-level shader; b_shader_target = true)
	 *		  note this function doesn't check for source code availability in
	 *		  supplied shaders (i.e. it's possible low-level shading language is supported
	 *		  but supplied shaders are available in high-level shading language only)
	 *		- returns false in case (all) required extensions aren't supported
	 */
	bool b_ShaderSupport(bool b_shader_target) const;

	/*
	 *	bool CGLShaderObject::b_ShaderAvailability(bool b_shader_target) const
	 *		- returns true in case shaders in language, specified by b_shader_target are
	 *		  available (returns false in case no shader was specified)
	 *		- b_shader_target specifies shader language (low-level program;
	 *		  b_shader_target = false or high-level shader; b_shader_target = true)
	 *		  note this function doesn't check for OpenGL support
	 *		- returns false in case (all specified) shaders are not available
	 *		  in a given language
	 */
	bool b_ShaderAvailability(bool b_shader_target) const;

	/*
	 *	bool CGLShaderObject::b_Status() const
	 *		- returns true in case vertex and pixel shaders are compatible
	 *		  so it's possible to compile them; otherwise returns false
	 */
	bool b_Status() const;

	/*
	 *	bool CGLShaderObject::b_ShaderTarget() const
	 *		- returns true in case high level shading language is going to be used
	 *		- returns false in case low level shading language is going to be used
	 */
	bool b_ShaderTarget() const;

	/*
	 *	void CGLShaderObject::Prefer_ShaderTarget(bool b_shader_target)
	 *		- hint to use low (b_shader_target = false) or high (b_shader_target = true)
	 *		  level shading language
	 *		- in case shaders are present in desired shading  language, it is going
	 *		  to be used. otherwise available shading language is going to be used
	 */
	void Prefer_ShaderTarget(bool b_shader_target);

	/*
	 *	bool CGLShaderObject::Compile_Link(CGLState *p_state, char *&r_p_s_compile_log,
	 *		char *&r_p_s_link_log)
	 *		- compile and link shaders
	 *		- p_state is active opengl state guard
	 *		- r_p_s_compile_log is compilation log, (compiller error / warnings output)
	 *		  r_p_s_link_log is linkage log, (linker error / warnings output)
	 *		  in case compiller / linker generated error / warning (text) output,
	 *		  string is allocated and pointer is put here. (it has to be freed) otherwise
	 *		  the pointer is set to 0
	 *		- returns true in case shaders were successfuly compiled and linked
	 *		  (note despite that warnings could have been generated, see log string pointers)
	 */
	bool Compile_Link(CGLState *p_state, char *&r_p_s_compile_log, char *&r_p_s_link_log);

	/*
	 *	CShaderVectorParameterRef<float> *CGLShaderObject::p_Find_Parameter(
	 *		const char *p_s_parameter_name)
	 *		- find float shader parameter by name p_s_parameter_name
	 *		- returns pointer to shader parameter object on success, 0 on failure
	 *		  (in case shaders weren't compiled or parameter with p_s_parameter_name doesn't exist)
	 */
	CShaderVectorParameterRef<float> *p_Find_Parameter(const char *p_s_parameter_name);

	/*
	 *	CShaderVectorParameterRef<int> *CGLShaderObject::p_Find_IntParameter(
	 *		const char *p_s_parameter_name)
	 *		- find integer shader parameter by name p_s_parameter_name
	 *		- returns pointer to shader parameter object on success, 0 on failure
	 *		  (in case shaders weren't compiled or parameter with p_s_parameter_name doesn't exist)
	 *		- note integer shader parameters are available in high-level shading language only
	 */
	CShaderVectorParameterRef<int> *p_Find_IntParameter(const char *p_s_parameter_name);

	/*
	 *	CShaderVectorParameterRef<int> *CGLShaderObject::p_Find_BoolParameter(
	 *		const char *p_s_parameter_name)
	 *		- find boolean shader parameter by name p_s_parameter_name
	 *		- returns pointer to shader parameter object on success, 0 on failure
	 *		  (in case shaders weren't compiled or parameter with p_s_parameter_name doesn't exist)
	 *		- note boolean shader parameters are available in high-level shading language only
	 */
	CShaderVectorParameterRef<int> *p_Find_BoolParameter(const char *p_s_parameter_name);

	/*
	 *	CShaderMatrixParameterRef *CGLShaderObject::p_Find_MatrixParameter(
	 *		const char *p_s_parameter_name)
	 *		- find matrix shader parameter by name p_s_parameter_name
	 *		- returns pointer to shader parameter object on success, 0 on failure
	 *		  (in case shaders weren't compiled or parameter with p_s_parameter_name doesn't exist)
	 *		- note matrix shader parameters are available in high-level shading language only
	 */
	CShaderMatrixParameterRef *p_Find_MatrixParameter(const char *p_s_parameter_name);

	/*
	 *	int CGLShaderObject::n_TextureType(const char *p_s_sampler_name) const
	 *		- find texture type by name p_s_sampler_name
	 *		- returns sampler type (one of shader_sampler_1D, shader_sampler_2D, shader_sampler_3D,
	 *		  shader_sampler_Cube, shader_sampler_1D_Shadow, shader_sampler_2D_Shadow,
	 *		  shader_sampler_2D_Rect, shader_sampler_2D_Rect_Shadow) or 0 on failure
	 *		- note samplers are available in high-level shading language only
	 */
	int n_TextureType(const char *p_s_sampler_name) const;

	/*
	 *	int CGLShaderObject::n_TextureUnit(const char *p_s_sampler_name) const
	 *		- find texture unit by name p_s_sampler_name
	 *		- returns texture unit index (0 to GL_MAX_TEXTURE_UNITS - 1) or -1 on failure
	 */
	int n_TextureUnit(const char *p_s_sampler_name) const;

	/*
	 *	bool CGLShaderObject::Bind(CGLState *p_state) const
	 *		- bind shader, set supplied parameters
	 *		- returns true on success, false on failure
	 */
	bool Bind(CGLState *p_state) const;

	/*
	 *	void CGLShaderObject::Release(CGLState *p_state) const
	 *		- release shader
	 */
	void Release(CGLState *p_state) const;

	/*
	 *	int CGLShaderObject::n_Api() const
	 *		- return id of api being used (shader_api_ARB_program,
	 *		  shader_api_ARB_shader or shader_api_Core_shader)
	 */
	int n_Api() const;

	/*
	 *	void CGLShaderObject::Delete_Shaders(CGLState *p_state)
	 *		- delete opengl shader objects (in case they were created)
	 */
	void Delete_Shaders(CGLState *p_state);

private:
	/*
	 *	void CGLShaderObject::Remove_Parameter(CShaderParameterRef<float> *p_parameter,
	 *		bool b_invalidate = true)
	 *		- remove parameter p_parameter from the list
	 *		- in case b_invalidate is true, p_parameter->Invalidate() is going to be called
	 *		  (to avoid infinite recurse and "zombie" parameters)
	 */
	void Remove_Parameter(CShaderParameterRef<float> *p_parameter, bool b_invalidate = true);

	/*
	 *	void CGLShaderObject::Remove_Parameter(CShaderParameterRef<int> *p_parameter,
	 *		bool b_invalidate = true)
	 *		- remove parameter p_parameter from the list
	 *		- in case b_invalidate is true, p_parameter->Invalidate() is going to be called
	 *		  (to avoid infinite recurse and "zombie" parameters)
	 */
	void Remove_Parameter(CShaderParameterRef<int> *p_parameter, bool b_invalidate = true);

	/*
	 *	void CGLShaderObject::Remove_Parameter(CShaderParameterRef<bool> *p_parameter,
	 *		bool b_invalidate = true)
	 *		- remove parameter p_parameter from the list
	 *		- in case b_invalidate is true, p_parameter->Invalidate() is going to be called
	 *		  (to avoid infinite recurse and "zombie" parameters)
	 */
	void Remove_Parameter(CShaderParameterRef<bool> *p_parameter, bool b_invalidate = true);

protected:
	bool Resolve_Uniforms();
	bool Fill_Parameters();

	template <class CShaderClass>
	class CParamBinder {
	protected:
		const CShaderClass *m_p_shader;
		CGLState *m_p_state;
		bool m_b_success;

	public:
		__inline CParamBinder(CGLState *p_state, const CShaderClass *p_shader)
			:m_p_state(p_state), m_p_shader(p_shader), m_b_success(true)
		{ }

		__inline operator bool() const
		{
			return m_b_success;
		}
	};

	template <class CShaderClass>
		class CSamplerBinder : public CParamBinder<CShaderClass> {
	public:
		__inline CSamplerBinder(CGLState *p_state, CShaderClass *p_shader)
			:CParamBinder<CShaderClass>(p_state, p_shader)
		{ }

		__inline void operator ()(const TSamplerHandle &r_sampler_handle)
		{
			m_b_success &= m_p_shader->_SetParam1i(m_p_state,
				r_sampler_handle.n_register, r_sampler_handle.n_texture_unit);
		}
	};

	template <class CShaderClass>
	class CVectorParamBinder_Float : public CParamBinder<CShaderClass> {
	public:
		__inline CVectorParamBinder_Float(CGLState *p_state, const CShaderClass *p_shader)
			:CParamBinder<CShaderClass>(p_state, p_shader)
		{ }

		__inline void operator ()(const TVectorParamHandle &r_param_handle)
		{
			_ASSERTE(r_param_handle.n_type >= shader_param_Float &&
				r_param_handle.n_type <= shader_param_Float4);

			if(r_param_handle.n_type == shader_param_Float4) { // assume 4 will be mostly used
				m_b_success &= m_p_shader->_SetParam4fv(m_p_state,
					r_param_handle.n_register, r_param_handle.t_value.p_float_value);
			} else if(r_param_handle.n_type == shader_param_Float) {
				m_b_success &= m_p_shader->_SetParam1fv(m_p_state,
					r_param_handle.n_register, r_param_handle.t_value.p_float_value);
			} else if(r_param_handle.n_type == shader_param_Float2) {
				m_b_success &= m_p_shader->_SetParam2fv(m_p_state,
					r_param_handle.n_register, r_param_handle.t_value.p_float_value);
			} else /*if(r_param_handle.n_type == shader_param_Float3)*/ {
				m_b_success &= m_p_shader->_SetParam3fv(m_p_state,
					r_param_handle.n_register, r_param_handle.t_value.p_float_value);
			}
		}
	};

	template <class CShaderClass>
	class CVectorParamBinder_Float4 : public CParamBinder<CShaderClass> {
	public:
		__inline CVectorParamBinder_Float4(CGLState *p_state, const CShaderClass *p_shader)
			:CParamBinder<CShaderClass>(p_state, p_shader)
		{ }

		__inline void operator ()(const TVectorParamHandle &r_param_handle)
		{
			_ASSERTE(r_param_handle.n_type == shader_param_Float4);

			m_b_success &= m_p_shader->_SetParam4fv(m_p_state,
				r_param_handle.n_target, r_param_handle.n_register,
				r_param_handle.t_value.p_float_value);
		}
	};

	template <class CShaderClass>
	class CVectorParamBinder_Int : public CParamBinder<CShaderClass> {
	public:
		__inline CVectorParamBinder_Int(CGLState *p_state, const CShaderClass *p_shader)
			:CParamBinder<CShaderClass>(p_state, p_shader)
		{ }

		__inline void operator ()(const TVectorParamHandle &r_param_handle)
		{
			_ASSERTE(r_param_handle.n_type >= shader_param_Int &&
				r_param_handle.n_type <= shader_param_Int4);

			if(r_param_handle.n_type == shader_param_Int4) { // assume 4 will be mostly used
				m_b_success &= m_p_shader->_SetParam4iv(m_p_state,
					r_param_handle.n_register, r_param_handle.t_value.p_int_value);
			} else if(r_param_handle.n_type == shader_param_Int) {
				m_b_success &= m_p_shader->_SetParam1i(m_p_state,
					r_param_handle.n_register, *r_param_handle.t_value.p_int_value);
			} else if(r_param_handle.n_type == shader_param_Int2) {
				m_b_success &= m_p_shader->_SetParam2iv(m_p_state,
					r_param_handle.n_register, r_param_handle.t_value.p_int_value);
			} else /*if(r_param_handle.n_type == shader_param_Int3)*/ {
				m_b_success &= m_p_shader->_SetParam3iv(m_p_state,
					r_param_handle.n_register, r_param_handle.t_value.p_int_value);
			}
		}
	};

	template <class CShaderClass>
	class CVectorParamBinder_Bool : public CParamBinder<CShaderClass> {
	public:
		__inline CVectorParamBinder_Bool(CGLState *p_state, const CShaderClass *p_shader)
			:CParamBinder<CShaderClass>(p_state, p_shader)
		{ }

		__inline void operator ()(const TVectorParamHandle &r_param_handle)
		{
			_ASSERTE(r_param_handle.n_type >= shader_param_Bool &&
				r_param_handle.n_type <= shader_param_Bool4);

			if(r_param_handle.n_type == shader_param_Bool4) { // assume 4 will be mostly used
				m_b_success &= m_p_shader->_SetParam4iv(m_p_state,
					r_param_handle.n_register, r_param_handle.t_value.p_int_value);
			} else if(r_param_handle.n_type == shader_param_Bool) {
				m_b_success &= m_p_shader->_SetParam1i(m_p_state,
					r_param_handle.n_register, *r_param_handle.t_value.p_int_value);
			} else if(r_param_handle.n_type == shader_param_Bool2) {
				m_b_success &= m_p_shader->_SetParam2iv(m_p_state,
					r_param_handle.n_register, r_param_handle.t_value.p_int_value);
			} else /*if(r_param_handle.n_type == shader_param_Bool3)*/ {
				m_b_success &= m_p_shader->_SetParam3iv(m_p_state,
					r_param_handle.n_register, r_param_handle.t_value.p_int_value);
			}
		}
	};

	template <class CShaderClass>
	class CMatrixParamBinder : public CParamBinder<CShaderClass> {
	public:
		__inline CMatrixParamBinder(CGLState *p_state, const CShaderClass *p_shader)
			:CParamBinder<CShaderClass>(p_state, p_shader)
		{ }

		__inline void operator ()(const TMatrixParamHandle &r_param_handle)
		{
			_ASSERTE(r_param_handle.n_type >= shader_param_Matrix2 &&
				r_param_handle.n_type <= shader_param_Matrix4);

			if(r_param_handle.n_type == shader_param_Matrix3) { // assume 3 will be mostly used
				m_b_success &= m_p_shader->_SetParamMatrix3fv(m_p_state,
					r_param_handle.n_register, false, r_param_handle.p_value);
			} else if(r_param_handle.n_type == shader_param_Matrix2) {
				m_b_success &= m_p_shader->_SetParamMatrix2fv(m_p_state,
					r_param_handle.n_register, false, r_param_handle.p_value);
			} else /*if(r_param_handle.n_type == shader_param_Matrix4)*/ {
				m_b_success &= m_p_shader->_SetParamMatrix4fv(m_p_state,
					r_param_handle.n_register, false, r_param_handle.p_value);
			}
		}
	};
};

#endif //__SHADER_SYSTEM_INCLUDED
